-- Grosse Uebung GUe5: Implementierung und Test einer UART
-- Datei: uart.vhd
-- Rahmenprogramm fuer den Test des UART-Empfaengers und -Senders
-- letzte Aenderung: 14.06.2016
-- Autor: C. Giesemann

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity uart is
 port (
        GCLK: in std_logic;						-- 100MHz Takt von der Baugruppe
        RXD : in std_logic;       				-- serieller Eingang vom PC (bzw. vom USB-Seriell-Umsetzer)
        TXD : out std_logic;					-- serieller Ausgang zum PC (bzw. vom USB-Seriell-Umsetzer)
		SW  : in std_logic_vector(7 downto 0);	-- 8 Schalter auf der Baugruppe
        LD  : out std_logic_vector(7 downto 0); -- 8 Leuchtdioden auf der Baugruppe
		BTNR: in std_logic;                     -- Taster fr "right" als Reset-Taster
        LA  : out std_logic_vector(3 downto 0));-- Ausgaenge zum Logikanalysator
end uart;

architecture Behavioral of uart is

signal clk_16x_baud, clk_baud : std_logic;
signal start, busy_sender, busy_receiver, busy_receiver_del : std_logic;
signal RXD_int, TXD_int : std_logic;

begin

sender: entity work.uart_sender 
 port map(start=>start, T=>clk_baud, I=>BTNR, dat=>SW, TxD=>TXD_int, busy=>busy_sender); 

receiver: entity work.uart_receiver 
 port map(T=>clk_16x_baud, I=>BTNR, RxD=>RXD_int, dat=>LD, busy=>busy_receiver); 


    -- Taktteiler fuer den Empfaengertakt (16-fachen Baudrate)
    process(GCLK)
        variable counter : integer range 0 to 333 := 333;
    begin
        if rising_edge(GCLK) then
            if counter = 0 then
                clk_16x_baud <= not clk_16x_baud;
                counter := 333;
            else
                counter := counter - 1;
            end if;
        end if;
    end process;
	 
	 -- Taktteiler fuer den Sendertakt (einfache Baudrate)
    process(clk_16x_baud)
        variable cnt : integer range 0 to 7 := 0;
    begin
        if rising_edge(clk_16x_baud) then
            if cnt < 7 then
                cnt := cnt + 1;
				else
                clk_baud <= not clk_baud;
					 cnt := 0;
            end if;
        end if;
    end process;
	 
	 -- Automat, der bei jedem Datenempfang eine Sendeoperation startet 
    process(clk_16x_baud)
    begin
        if rising_edge(clk_16x_baud) then
            if busy_receiver = '1' and busy_receiver_del = '0' then
               start <= '1';
				elsif start = '1' and busy_sender = '1' then 
					start <= '0';
            end if;
		  busy_receiver_del <= busy_receiver;
        end if;
    end process;
	 
	 	 

RXD_int <= RXD;
TXD <= TXD_int;
LA(0) <= RXD_int;
LA(1) <= busy_receiver;
LA(2) <= TXD_int;
LA(3) <= busy_sender;

end Behavioral;

